using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.IO;

using MapApp.Model;
using MapApp.Ui.Controls;

namespace MapApp.Ui.Controls
{
    public partial class FrmMain : Form
    {
        private static string MAP_LOCATIONS_FILE_PATH = AppUtil.TempHtmlDir + "MapLocations";
        private static string HELP_FILE_PATH = Application.StartupPath + @"\MapApp.chm";

        FrmMarkers frmMarkers = null;
        FrmOptions frmOptions = null;
        FrmInfo frmInfo = null;
        FrmStreetView frmStreetView = null;
        FrmFind frmFind = null;
        FrmAbout frmAbout = null;
        public static FrmMain Instance;

        public void SetStatusText(string value)
        {
            statusLabel.Text = value;
        }

        public void SetCoordText(string value)
        {
            coordLabel.Text = value;
        }

        public FrmMain()
        {
            InitializeComponent();
            if (!DesignMode)
                CustomInitialize();
        }

        private void CustomInitialize()
        {
            AppUtil.Init(this);
            FrmMain.Instance = this;
            ////this.frmOptions = new FrmOptions();
            this.frmFind = new FrmFind();
            this.frmAbout = new FrmAbout();

            MapLocation initialCenterAndZoom = CenterAndZoomFromString(Properties.Settings.Default.Map_CenterAndZoom);
            mapControl.InitialCenterAndZoom = initialCenterAndZoom;
            int initialMapType = Properties.Settings.Default.MapType;
            mapControl.InitialMapType = initialMapType;

            try
            {
                FileStream inStream = new FileStream(MAP_LOCATIONS_FILE_PATH, FileMode.Open);
                Map.MapLocations = MapLocationCollection.Load(inStream);
                inStream.Close();
            }
            catch { }

            mapControl.DisplayMap();

            mapControl.MapMouseMove += new GoogleMapControl.OnMapMouseMove(mapControl_MapMouseMove);
            mapControl.MapClick += new GoogleMapControl.OnMapClick(mapControl_MapClick);
            mapControl.ShowStreetView += new GoogleMapControl.OnShowStreetView(mapControl_ShowStreetView);

            StartPosition = FormStartPosition.Manual;
            Location = Properties.Settings.Default.FrmMain_Location;
            Size = Properties.Settings.Default.FrmMain_Size;
            //Size frmMainSize = Properties.Settings.Default.FrmMain_Size;
            //if (frmMainSize.Width == 0) frmMainSize.Width = 400;
            //if (frmMainSize.Height == 0) frmMainSize.Height = 400;
            //Size = frmMainSize;
            WindowState = Properties.Settings.Default.FrmMain_WindowState;
            if (WindowState == FormWindowState.Minimized)
            {
                WindowState = FormWindowState.Normal; // never start a window minimized
            }

#if DEBUG
            streetViewToolStripMenuItem.Visible = true;
#else
            streetViewToolStripMenuItem.Visible = false;
#endif // DEBUG

        }


        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public Map Map
        {
            get { return MapControl.Map; }
        }

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public GoogleMapControl MapControl
        {
            get { return mapControl; }
        }


        //private Point _rightClickMousePosition;
        //private MapLocation _rightClickMapLocation;

        void mapControl_MapClick(GoogleMapControl sender, MapClickEventArgs e)
        {
            //    //string msg = "Lat: " + e.MapLocation.Lat +
            //    //        ", Lng: " + e.MapLocation.Lng;
            //    //AppUi.StatusText = e.Button + " click: " + msg;
            //    //if (e.Button == MouseButtons.Right)
            //    //{
            //    //    _rightClickMousePosition = e.MousePosition;
            //    //    _rightClickMapLocation = e.MapLocation;
            //    //    zoomToAllMarkersContextMenuItem.Visible = Map.MapLocations.Count > 0;
            //    //    contextMenuStrip.Show(sender, e.MousePosition);
            //    //}
        }

        void mapControl_MapMouseMove(MapApp.Ui.Controls.GoogleMapControl sender, MapLocation loc)
        {
            string msg = String.Format("Lat: {0:n2}, Lng: {1:n2}, Zoom: {2:n0}", loc.Lat, loc.Lng, loc.ZoomLevel );
            AppUi.CoordText = msg;
        }

        void mapControl_ShowStreetView(GoogleMapControl sender, MapLocation loc)
        {
            if (frmStreetView == null)
            {
                frmStreetView = new FrmStreetView(loc);
            }
            frmStreetView.StreetViewControl.SetLocation(loc);
            frmStreetView.Show();
        }


        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Hide();
            this.Dispose();
        }



        private void optionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (frmOptions == null)
            {
                frmOptions = new FrmOptions();
            }
            frmOptions.Show();

        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmAbout.Show();
        }

        private void findToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmFind.Show();
        }

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        private static MapLocation CenterAndZoomFromString(string str)
        {
            MapLocation result = new MapLocation("", 0.0, 0.0, 0);
            try
            {
                string[] sArray = str.Split(':');
                result.Lat = Convert.ToDouble(sArray[0]);
                result.Lng = Convert.ToDouble(sArray[1]);
                result.ZoomLevel = Convert.ToInt32(sArray[2]);
            }
            catch { } // ignore exceptions
            return result;
        }

        private static string CenterAndZoomToString(MapLocation loc)
        {
            string result = "";
            if (loc == null) return result;
            result = loc.Lat + ":" + loc.Lng + ":" + loc.ZoomLevel;
            return result;
        }

        private void FrmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.Hide();

            try
            {
                FileStream outStream = new FileStream(MAP_LOCATIONS_FILE_PATH, FileMode.Create);
                Map.MapLocations.Save(outStream);
                outStream.Close();
            }
            catch { }


            Properties.Settings.Default.Map_CenterAndZoom = CenterAndZoomToString(MapControl.GetCenterAndZoom());
            Properties.Settings.Default.MapType = this.MapControl.GetMapType();

            if (WindowState == FormWindowState.Normal)
            {
                Properties.Settings.Default.FrmMain_Location = Location;
                Properties.Settings.Default.FrmMain_Size = Size;
            }
            else
            {
                Properties.Settings.Default.FrmMain_Location = RestoreBounds.Location;
                Properties.Settings.Default.FrmMain_Size = RestoreBounds.Size;
            }
            Properties.Settings.Default.FrmMain_WindowState = WindowState;
            Properties.Settings.Default.Save();

            if (frmAbout != null)
            {
                frmAbout.Hide();
                frmAbout.Dispose();
            }
            if (frmOptions != null)
            {
                frmOptions.Hide();
                frmOptions.Dispose();
            }
            if (frmInfo != null)
            {
                frmInfo.Hide();
                frmInfo.Dispose();
            }
            if (frmMarkers != null)
            {
                frmMarkers.Hide();
                frmMarkers.Dispose();
            }
            if (frmFind != null)
            {
                frmFind.Hide();
                frmFind.SaveHistoryFile();
                frmFind.Dispose();
            }

            /// Delete TempHtmlDir (which is unique to this process):
            AppUtil.DeleteTempHtmlDir();
        }

        private void printPreviewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //Bitmap bmp = new Bitmap(mapControl.Width, mapControl.Height);
            //mapControl.DrawToBitmap(bmp, new Rectangle(mapControl.Location.X, mapControl.Location.Y,
            //    mapControl.Width, mapControl.Height));

            //Form dummy = new Form();
            //dummy.BackgroundImage = bmp;
            //dummy.Show();
            MapControl.ShowPrintPreviewDialog();
        }

        private void markersToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (frmMarkers == null)
            {
                frmMarkers = new FrmMarkers();
                frmMarkers.MapLocations = MapControl.Map.MapLocations;
                frmMarkers.MapControl = MapControl;
            }
            frmMarkers.Show();
        }

        private void infoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (frmInfo == null)
            {
                frmInfo = new FrmInfo();
            }
            frmInfo.Show();
        }

        private void streetViewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (frmStreetView == null)
            {
                frmStreetView = new FrmStreetView();
            }
            frmStreetView.Show();

        }

        private void printToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MapControl.Print();
        }

        private void contentsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Help.ShowHelp(this, HELP_FILE_PATH, HelpNavigator.TableOfContents);
        }

        private void indexToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Help.ShowHelpIndex(this, HELP_FILE_PATH);
        }

        private void searchToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Help.ShowHelp(this, HELP_FILE_PATH, HelpNavigator.Find, "");
        }

        private void keyboardShortcutsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Help.ShowHelp(this, HELP_FILE_PATH, HelpNavigator.TopicId, "4");
        }

        private void undoZoomToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MapControl.UndoZoom();
        }

        private void redoZoomToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MapControl.RedoZoom();
        }



        //private void FrmMain_ResizeBegin(object sender, EventArgs e)
        //{
        //    try
        //    {
        //        _centerBeforeResize = MapControl.GetCenterAndZoom();
        //    }
        //    catch { } // ignore exception - MapControl might not yet exist
        //    _windowStateBeforeResize = this.WindowState;
        //}

        //private void FrmMain_ResizeEnd(object sender, EventArgs e)
        //{
        //    if (_windowStateBeforeResize != WindowState &&
        //        _centerBeforeResize != null)
        //    {
        //        MapControl.SetCenterAndZoom(_centerBeforeResize);
        //    }
        //}

        //private MapLocation _centerBeforeResize;
        //private FormWindowState _windowStateBeforeResize;

        //private void FrmMain_Resize(object sender, EventArgs e)
        //{
        //    switch (WindowState)
        //    {
        //        case FormWindowState.Maximized:
        //        case FormWindowState.Normal:
        //            if (_windowStateBeforeResize != WindowState &&
        //                _windowStateBeforeResize != FormWindowState.Minimized &&
        //                _centerBeforeResize != null)
        //            {
        //                MapControl.SetCenterAndZoom(_centerBeforeResize);
        //            }
        //            break;
        //    }
        //    // set old values for next time:
        //    try
        //    {
        //        _centerBeforeResize = MapControl.GetCenterAndZoom();
        //        AppUi.StatusText = "Center before resize: " + _centerBeforeResize;
        //    }
        //    catch { } // ignore exception - MapControl might not yet exist
        //    _windowStateBeforeResize = this.WindowState;
        //}




    }
}